# PL/SQL 中的异常处理

> 原文：<https://www.studytonight.com/plsql/plsql-exception-handling>

异常是在执行 PL/SQL 代码块期间发生的运行时错误或意外事件。

oracle 引擎是第一个识别这种异常的引擎，它会立即尝试通过默认的异常处理程序来解决它。

默认异常处理程序是内存中预定义的代码块，用于对异常采取适当的操作。

异常处理可以在 PL/SQL 程序代码块的**异常**部分完成。

以下是它的语法:

```sql
DECLARE
	-- Declaration statements;
BEGIN
	-- SQL statements;
	-- Procedural statements;
EXCEPTION
	-- Exception handling statements;
END;
```

### 有两种类型的例外:

1.  系统(预定义)例外
2.  用户定义的例外

让我们逐一介绍这两种类型的异常。

* * *

## 系统(预定义)例外

为了处理运行 PL/SQL 代码时出现的常见异常，oracle 中有两种类型的异常处理程序:

*   命名异常处理程序
*   编号异常处理程序

### 命名异常处理

这些异常是 oracle 为最常见的异常预先定义的名称。

以下是处理命名异常的语法:

```sql
EXCEPTION
	WHEN <exception_name> THEN
		-- take action 
```

默认情况下，有许多预定义的命名异常可用。下表中几乎没有显示它们及其含义:

| 命名异常 | 意义 |
| `LOGIN_DENIED` | 当连接到 Oracle 时给出无效的用户名或密码时发生。 |
| `TOO_MANY_ROWS` | 当 select 语句返回多行时发生。 |
| `VALUE_ERROR` | 当用户给出无效的数据类型或大小时发生。 |
| `NO_DATA_FOUND` | 当找不到记录时发生。 |
| `DUP_VAL_ON_INDEX` | 当对某一列应用了唯一约束，并且执行“插入”或“更新”导致为该列创建重复记录时发生。 |
| `PROGRAM_ERROR` | 当程序中出现内部错误时发生。 |
| `ZERO_DIVIDE` | 当任何变量值除以零时发生。 |

**时间为例！**

下面我们有一个简单的 PL/SQL 代码块，用来演示命名异常处理程序的使用，

```sql
set serveroutput on;

DECLARE
	a int;
	b int;
	c int;
BEGIN
	a := &a;
	b := &b;
	c := a/b;
	dbms_output.put_line('RESULT=' || c);
EXCEPTION
	when ZERO_DIVIDE then
		dbms_output.put_line('Division by 0 is not possible');
END;
```

输入 a:10 的值输入 b:0 的值除以 0 是不可能的 PL/SQL 过程成功完成。

### 编号异常处理

在甲骨文中，一些预定义的异常以**四个整数的形式编号，前面有一个连字符符号**。为了处理这样的异常，我们应该在使用它们之前给它们指定一个名称。

这可以通过使用 **Pragma 异常技术**来实现，在该技术中，编号异常处理程序绑定到一个名称。为此，我们在 PL/SQL 程序中使用一个关键字，并使用以下语法编写一个将名称绑定到编号异常的语句，该语句写在程序的 **DECLARE** 部分:

```sql
pragma exception_init(exception_name, exception _number);
```

其中，`pragma exception_init`(大小写无关紧要)是表示 Pragma 异常技术的关键字，有两个参数:

*   **异常 _ 名称**，这是一个用户定义的名称，如果发生了预定义的编号异常，则赋予该名称。
*   **exception_number** ，是甲骨文分配给该异常的编号。

**时间为例！**

下面我们有一个表格，里面有学生的数据。

| 滚装船 | SNAME | 年龄 | 课程 |
| Eleven | 安奴 | Twenty | 平衡计分卡 |
| Twelve | 阿莎 | Twenty-one | 商学士 |
| Thirteen | 阿坡 | Eighteen | 喹啉甲酸 |
| Fourteen | 男性 | Twenty | 喹啉甲酸 |
| Fifteen | 波动 | Nineteen | BBA |

在下面的 PL/SQL 程序中，我们将使用上表学生演示编号异常的使用，

```sql
set serveroutput on;

DECLARE
	sno student.rollno%type;
	snm student.sname%type;
	s_age student.age%type;
	cr student.course%type;
	-- Exception name declared below
	already_exist EXCEPTION;
	-- pragma statement to provide name to numbered exception
	pragma exception_init(already_exist, -1);
BEGIN
	sno:=&rollno;
	snm:='&sname';
	s_age:=&age;
	cr:='&course';
	INSERT into student values(sno, snm, s_age, cr);
	dbms_output.put_line('Record inserted');
	EXCEPTION
		WHEN already_exist THEN
			dbms_output.put_line('Record already exist');
END;
```

输入 sno:11 的值输入 snm:heena 的值输入 s_age:20 的值输入 cr:bsc 记录的值已存在 PL/SQL 过程已成功完成。

在上述程序中，只要违反了主键概念(记录应唯一且不为空)，oracle 就会通过 **-1** 生成一个编号异常，这就是为什么在执行上述程序期间用户输入的**rollino**为 **11** 的原因。程序的异常部分开始运行，并在用户**记录已经存在**之前显示消息。

使用程序**的**声明**部分中的`pragma`关键字已经存在**字符串被映射到一个编号的异常 **-1** 。

* * *

## 用户定义的异常

在任何程序中，都有可能出现一些 oracle 不认为是异常的错误。在这种情况下，程序员可以在编写代码时定义异常。这种类型的异常称为用户定义异常。

用户定义的异常通常被定义为处理特殊情况，在这种情况下，由于我们的代码逻辑，我们的代码可以生成异常。

此外，在代码逻辑中，您可以使用`RAISE`关键字显式指定生成异常，然后使用`EXCEPTION`块处理它。

下面是它的语法，

```sql
DECLARE
	<exception name> EXCEPTION
BEGIN
	<sql sentence>
	If <test_condition> THEN 
		RAISE <exception_name>;
	END IF;
	EXCEPTION
		WHEN <exception_name> THEN
			-- some action
END;
```

让我们举个例子来理解如何使用用户定义的异常。下面我们有一个简单的例子，

| 滚装船 | SNAME | 总计 _ 课程 |
| Eleven | 安奴 | Two |
| Twelve | 阿莎 | one |
| Thirteen | 阿坡 | three |
| Fourteen | 男性 | one |

在下面的 PL/SQL 程序中，我们将使用上表学生演示用户定义异常的使用，

```sql
set serveroutput on;

DECLARE
	sno student.rollno%type;
	snm student.sname%type;
	crno student.total_course%type;
	invalid_total EXCEPTION;
BEGIN
	sno := &rollno;
	snm := '&sname';
	crno:=total_courses;
	IF (crno > 3) THEN 
		RAISE invalid_total;
	END IF;
	INSERT into student values(sno, snm, crno);
	EXCEPTION
		WHEN invalid_total THEN
			dbms_output.put_line('Total number of courses cannot be more than 3');
END;
```

输入 sno:15 的值输入 snm:Akash 的值输入 crno:5 的值课程总数不能超过 3 成功完成的 PL/SQL 过程。

在上面的程序中，

使用名为 **invalid_total** 的自定义异常，该异常在课程总数大于 3(一个学生最多可注册 3 门课程)时生成

这样我们就没有例外了。

* * *