# Kotlin 构造器——主构造器和辅助构造器

> 原文：<https://www.studytonight.com/kotlin/kotlin-constructor-primary-and-secondary-constructor>

在本教程中，我们将讨论 Kotlin 类构造器。创建类对象时，构造器用于初始化类属性。它可以被认为是一种特殊的功能。构造器在**对象创建**时被调用。每当我们创建一个类的对象时，都会自动调用构造器。

Kotlin 中有两种类型的构造器:

1.  主构造器

2.  二级建造师

只能有**一个一级建造师****多个二级建造师**。

## Kotlin 主构造器

Kotlin 最重要的特点之一是简洁。从主构造器的声明可以看出。主构造器是类头的一部分。类的所有属性也成为了主构造器的一部分。

通过在类名末尾添加`constructor()`来创建主构造器:

```kt
class ClassName constructor(){
}
```

也可以删除`constructor`关键字:

```kt
class ClassName(){
}
```

这意味着，当我们定义一个普通类时，主承包商被创建。

让我们为在上一个教程中创建的 **Mobile** 类创建一个主构造器:

```kt
class Mobile (var brand: String, var model: String, var mrp: Float, var discount: Float) {

    // class methods
    fun getActualPrice():Float{
        return mrp - discount
    }

    fun printDetails(){
        println("Mobile details:")
        println("Brand: $brand")
        println("Model: $model")
        println("MRP: $mrp")
        println("Discount: $discount")

    }
} 
```

这里所有的类属性都被定义为主构造器的参数。

现在，让我们创建一个这个类的对象:

```kt
fun main() {
    val mobile: Mobile = Mobile("iPhone", "11 pro", 100000f, 1000f)
    println("Discounted price is: ${mobile.getActualPrice()}")
    mobile.printDetails()
}
```

折扣价为:99000.0
手机详情:
品牌:iPhone
型号:11 pro
MRP: 100000.0
折扣:1000.0

现在变量由主构造器初始化。

但是有必要提供类属性的所有值。为此，我们还可以向主构造器提供默认值:

```kt
class Mobile (var brand: String = "", var model: String = "", var mrp: Float = 0f, var discount: Float = 0f)
```

## 带`init`块的主构造器

在主构造器中，提供的值直接分配给类属性。如果我们想在赋值之前改变这些值，或者**给主构造器添加一些逻辑**，我们可以使用`init`块。

下面是一个代码示例，

```kt
class Mobile constructor(brand: String, model: String, mrp: Float, discount: Float) {
    var brand: String
    var model: String
    var mrp: Float
    var discount: Float

    // init function
    init {
        println("In init")
        this.brand = brand.toUpperCase()
        this.model = model.toUpperCase()
        this.mrp = mrp
        this.discount = discount
    }

    fun getActualPrice():Float{
        return mrp - discount
    }

    fun printDetails(){
        println("Mobile details:")
        println("Brand: $brand")
        println("Model: $model")
        println("MRP: $mrp")
        println("Discount: $discount")

    }
} 
```

现在我们可以创建对象并打印详细信息:

```kt
fun main() {
    val mobile: Mobile = Mobile("iPhone", "11 pro", 100000f, 1000f)
    println("Discounted price is: ${mobile.getActualPrice()}")
    mobile.printDetails()
}
```

In init
折扣价为:99000.0
手机详情:
品牌:IPHONE
型号:11 PRO
MRP: 100000.0
折扣:1000.0

关于`init`区块的要点:

*   `init`块总是在主构造器之后调用。

*   一个类可以有多个`init`块。

## Kotlin 二级建造师

辅助构造器用于初始化一组值。使用`constructor`关键字创建辅助构造器。一个类可以有**一个或多个二级构造器**。

让我们在 **Mobile** 类中创建两个构造器:

```kt
class Mobile {
    var brand: String = ""
    var model: String = ""
    var mrp: Float = 0f
    var discount: Float = 0f

    // first secondary constructor
    constructor(_brand: String, _model: String){
        this.brand = _brand
        this.model = _model
    }

    // second secondary constructor
    constructor(_mrp: Float, _discount: Float){
        this.mrp = _mrp
        this.discount = _discount
    }

    fun getActualPrice():Float{
        return mrp - discount
    }

    fun printDetails(){
        println("Mobile details:")
        println("Brand: $brand")
        println("Model: $model")
        println("MRP: $mrp")
        println("Discount: $discount")

    }
}
```

现在我们可以通过根据构造器传递值来创建一个对象:

```kt
fun main() {
    val mobile: Mobile = Mobile("iPhone", "11 pro")
    println("Discounted price is: ${mobile.getActualPrice()}")
    mobile.printDetails()
}
```

折扣价为:0.0
手机详情:
品牌:iPhone
型号:11 pro
MRP: 0.0
折扣:0.0

关于二级构造器的几点:

*   二级建造师**没那么常见**。它们通常被避免，在对象创建之前，`init`块可以用来执行任何特定的任务。

*   If the Primary constructor is already present then each secondary constructor should call the primary constructor. It is called using **this()**:

    ```kt
    constructor(_brand: String, _model: String): this()
    ```

*   我们也可以使用 **this()** 从另一个二级构造器调用一个二级构造器:

    ```kt
    constructor(_brand: String, _model: String): this(10f,1f)
    ```

    该构造器正在调用其他具有值 **10.0** 和 **1.0** 的构造器用于`mrp`和`discount`。

*   我们也可以使用 **super()** 调用父类的构造器(在继承的情况下)。我们将在 Kotlin 继承教程中讨论它。

## 总结

在本教程中，我们讨论了 Kotlin 中的构造器。我们看到了主构造器、带`init`块的主构造器和次构造器。我们将在下一个教程中讨论伴随对象。

* * *

* * *