# Kotlin 继承

> 原文：<https://www.studytonight.com/kotlin/kotlin-inheritance>

在本教程中，我们将学习 Kotlin 中的继承概念，这是面向对象编程的一个主要特征。我们将学习如何在 Kotlin 的类中实现继承，主构造器和次构造器如何在继承中起作用，我们还将学习如何重写类的方法和属性。

### 什么是继承？

继承是面向对象编程最重要的特征之一。它允许一个类继承另一个类的特性。

假设您创建了一个类`Vehicle`，它具有与车辆相关的属性和方法。现在我们需要创建另一个类`Car`，它有许多车辆共有的属性。因此，我们可以在`Car`类中继承`Vehicle`类，而不是重新编写相同的代码。类似地，我们可以创建一个类`TwoWheeler`，它同样可以从 Vehicle 类继承一些基本的公共属性，并且可以有自己的一些属性。

通过这种方式，可以使用继承来正确地划分代码，从而实现代码的重用，并为代码提供更多的结构。

继承允许新类继承现有类的属性和方法。现有类或继承新类的类称为**基类**或**父类**或**超类**。继承已有类特征的新类称为**派生类**或**子类**或**子类**。

## Kotlin 的继承

Kotlin 的所有职业默认都是**`final`。这意味着这些**职业是不可继承的**。为了使类可继承，我们在类头中添加`open`关键字。**

 **子类使用`:`运算符继承父类。继承类的**语法是:**

```kt
open class Parent{
    // Parent class features
}
class Child : Parent(){
    // Child class features
}
```

让我们创建一个非常基本的继承示例:

```kt
// parent class
open class Vehicle{
    fun run(){
        println("Vehicle runs....")
    }
}
// child class
class Car : Vehicle(){
    fun seats(){
        println("Car has 4 seats")
    }
}

fun main() {
    val car = Car()
    car.seats()
    // calling method defined in parent class
    car.run()
}
```

汽车有 4 个座位
车辆行驶....

在这个例子中，我们已经将类`Vehicle`标记为开放，以使其可继承。类`Car`继承类`Vehicle`，从而继承其功能`run()`。

在 Kotlin 中，一个类只能继承一个类，这与 Java 相同。由此可见， **Kotlin 不允许多重继承**。但是一个**类可以实现很多接口**，我们将在接下来的教程中讨论。

## 继承中的主构造器

如果父类有主构造器，那么**子类必须调用父类**的主构造器。在上面的例子中，在`Car`类中，我们也调用了`Vehicle`类的主构造器(由于我们没有提供主构造器，**主构造器是由编译器**创建的)。让我们将属性传递给父类，看看它们是如何工作的。

```kt
/*
  Vehicla class has properties:
     - is the vehicle for carrying passengers
     - no. of tyres
     - fuel capacity of vehicle
*/
open class Vehicle(carryPassenger: Boolean, tyres: Int, fuelCapacity: Int){
    init {
        println("Does Vehicle carry passenger: $carryPassenger")
        println("Tyres in Vehicle: $tyres")
        println("Fuel capacity of Vehicle: $fuelCapacity")
    }
}

// child class with extra airBags property
class Car(carryPassenger: Boolean, tyres: Int, fuelCapacity: Int, airBags: Int) : Vehicle(carryPassenger, tyres, fuelCapacity){
    // init function for child class
    init {
        println("No. of air bags in car: $airBags")
    }
}

fun main() {
    val car = Car(true, 4, 50, 4)
}
```

车辆是否载客:真
车内轮胎:4
车内燃油容量:50
车内气囊数量:4

在这个例子中，我们在创建子类的对象时传递了值。从这些属性中，我们将`carryPassenger`、`tyres`、`fuelCapacity`传递给父类、`Vehicle`的**主构造器。**

创建`Car`类的对象时，调用子类 Car 的主构造器(默认)，子类 Car 内部调用**父类** `Vehicle`的主构造器，首先调用，然后调用**它的`init`块**。之后，子类`Car`的主默认构造器完成，然后执行子类 `Car`的`init` **块。**

## 继承中的辅助构造器

如果基类没有主构造器，但是有次构造器，那么派生类应该使用`super`关键字调用基类的次构造器。

```kt
open class Vehicle{
    // secondary constructor
    constructor(carryPassenger: Boolean, tyres: Int, fuelCapacity: Int){
        println("Does Vehicle carry passenger: $carryPassenger")
        println("Tyres in Vehicle: $tyres")
        println("Fuel capacity of Vehicle: $fuelCapacity")
    }
}
// child class
class Truck: Vehicle{
    // secondary constructor calling super for parent class
    constructor(carryPassenger: Boolean, tyres: Int, fuelCapacity: Int, loadCarryCapacity: Int): super(carryPassenger, tyres, fuelCapacity){
        println("Load carrying capacity is: $loadCarryCapacity")
    }
}

fun main() {
    val truck = Truck(false, 10, 200, 1000)
}
```

车辆载客吗:假
车内轮胎:10
车辆燃油量:200
载重量:1000

在本例中，`Truck`类的二级构造器使用`super`关键字调用`Vehicle`类的二级构造器。通过`super`调用时，会提供父类所需的所有参数。

## Kotlin 函数覆盖

在继承中，我们可以在派生类中提供基类函数的具体实现。这意味着如果一个函数存在于父类中，那么我们可以**在子类**中提供相同函数的不同实现。被称为**功能超越**。

在 Kotlin 中，默认情况下功能也是`final`**。要覆盖一个函数，它必须在父类中标记为`open`(就像类一样)，在子类中，我们必须使用关键字`override`而不是`open`来指定我们已经覆盖了该函数。**

 ****功能名称和参数应相同**。

并且，子类函数的**返回类型可以是父类函数的**子类型**。**

让我们看一个例子，其中我们将重写基类的两个方法:

```kt
// parent class
open class Vehicle{

    open fun run(){
        println("Vehicle is running")
    }

    open fun drivenBy(name: String){
        println("Vehicle is driven by: $name")
    }
}
// child class
class Jeep: Vehicle(){

    override fun run() {
        println("Driving a Jeep")
    }

    override fun drivenBy(name: String) {
        println("Jeep is driven by $name")
    }
}

fun main() {
    val jeep = Jeep()
    jeep.run()
    jeep.drivenBy("Ninja")
}
```

驾驶吉普车
吉普车由忍者驾驶

在这个例子中，我们覆盖了`Vehicle`类的两个方法`run()`和`drivenBy()`，并在`Jeep`类中覆盖了它们。

我们也可以使用`super`关键字在子类内部调用父类函数。

## Kotlin 覆盖属性

同样，我们也可以覆盖属性并**在子类**中更改它们的值。属性应该标记为`open`以便覆盖它们，并且当我们覆盖属性时，在子类中使用`override`关键字，就像我们覆盖类函数一样。

```kt
package inheritance

// parent class
open class Vehicle{
    open val color: String = "Black"
    open fun printColor(){
        println("The color of vehicle is: $color")
    }
}
// child class
class Jeep: Vehicle(){
    override val color: String = "Red"
    override fun printColor(){
        println("The color of Jeep is: $color")
    }
}

fun main() {
    val jeep = Jeep()
    jeep.printColor()
}
```

吉普的颜色是:红色

## 摘要

在本教程中，我们讨论了 Kotlin 中的继承、如何在类中创建父子关系、构造器在 inheritance 期间的行为以及方法和属性重写。在下一个教程中，我们将讨论 Kotlin 中的抽象类。

* * *

* * *****