# Kotlin 数据类

> 原文：<https://www.studytonight.com/kotlin/kotlin-data-class>

在本教程中，我们将介绍 Kotlin 中**数据类**的概念。在每个项目中，我们需要一些类，它们的**唯一目的是存储数据**。这些类被称为**数据类**或**数据访问对象**或**数据传输对象**等。

数据类主要包含:

*   变量及其获取者和设定者，

*   构造器，

*   一些功能如`toString()`、`equals()`、`hashCode()`等。

在 Java 中，我们需要编写大量的样板代码来定义数据类。让我们在 **Java** 中创建一个简单的`Person`类，它有一些属性。这就是 Java 中的代码:

```kt
public class Person {
    private String name;
    private String gender;
    private int age;

    public Person(String name, String gender, int age) {
        this.name = name;
        this.gender = gender;
        this.age = age;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getGender() {
        return gender;
    }

    public void setGender(String gender) {
        this.gender = gender;
    }

    public int getAge() {
        return age;
    }

    public void setAge(int age) {
        this.age = age;
    }

    @Override
    public String toString() {
        return "Person{" +
                "name='" + name + '\'' +
                ", gender='" + gender + '\'' +
                ", age=" + age +
                '}';
    }
} 
```

写这段代码真的很烦人，因为它只是保存数据并提供管理数据的实用功能。Kotlin 通过引入**数据类**解决了这个问题。

## Kotlin 数据类

使用`data`关键字创建数据类。默认情况下，Kotlin 数据类提供:

*   类属性的获取器和设置器(对于普通类也是如此)。

*   `toString()`功能打印对象的详细信息。

*   `copy()`将一个对象的内容复制到另一个对象的功能。

*   `equals()`和`hashCode()`函数用于散列和比较。

*   `componentN()`在构造器中按照属性的声明顺序对应的函数。

让我们在 Kotlin 中创建相同的数据类`Person`(就像我们在上面的 Java 中所做的那样):

```kt
data class Person(var name: String, var gender: String, var age: Int)
```

仅此而已。这是 Kotlin 简洁的一个很好的例子。

让我们**创建一个这个类的对象**并打印出来:

```kt
fun main() {
    val man = Person("Ninja", "Male", 18)
    println(man)
}
```

人(姓名=忍者，性别=男，年龄=18)

打印对象时，调用`toString()`函数本身。所以从 Kotlin 数据类的例子中，我们可以看到创建一个数据类然后使用它是多么容易，所有的基本函数都是由 Kotlin 自动定义的。

## 对数据类的要求

要创建数据类，应满足以下要求:

*   在主构造器中，至少应该有一个参数。

*   主构造器中的所有参数必须用`val`或`var`标记。

*   数据类不能是抽象的、开放的、密封的或内部的。

## 复制数据类中的对象属性

如果我们想为 Kotlin 中的任何数据类创建一个新对象，使用另一个对象的一些属性。然后我们可以使用`copy()`功能来实现:

```kt
fun main() {
    val man = Person("Ninja", "Male", 18)
    val woman = man.copy(gender = "Female")
    println(man)
    println(woman)
}
```

人(姓名=忍者，性别=男，年龄=18)
人(姓名=忍者，性别=女，年龄=18)

当我们使用`copy()`函数从任何对象复制单个属性值时，我们应该使用[命名参数](https://www.studytonight.com/kotlin/kotlin-positional-default-and-named-function-arguments)，否则该值可能会被分配给其他变量。

## Kotlin 数据类:`hashCode`和`equals`函数

`equals()`和`hashCode()`功能用于检查**两个物体是否相等**。`hashCode()`生成一个对象的哈希值。如果两个对象的所有对应值相等，则两个对象的哈希值相同。`equals()`功能也用于检查两个物体是否相等。

让我们举一个代码示例:

```kt
fun main() {
    val man = Person("Ninja", "Male", 18)
    val manNew = Person("Ninja", "Male", 18)
    val woman = man.copy(gender = "Female")
    if (man.hashCode() == manNew.hashCode())
        println("man and manNew are equal")
    if (man.hashCode() == woman.hashCode())
        println("It will not be printed")
    if (man.equals(manNew))
        println("man and manNew are equal")
    if (man.equals(woman))
        println("It will not be printed")
}
```

人和曼诺是平等的
人和曼诺是平等的

## 析构数据类对象- `componentN()`函数

如果我们想把一个对象的属性映射成独立变量，那么这是一个乏味的任务。在 Kotlin 中，我们可以使用 ***销毁声明*** 来销毁一个对象并获取其在自变量中的属性。让我们把`man`物体破坏成独立的变量:

```kt
fun main() {
    val man = Person("Ninja", "Male", 18)
    val (name,gender) = man
    println("Independent variable name: $name")
    println("Independent variable gender: $gender")
}
```

自变量名称:忍者
自变量性别:男

这是可能的，因为编译器将为每个属性生成`componentN()`函数，如:

```kt
man.component1()     // Ninja
man.component2()     // Male
man.component3()     // 18
```

因此，如果您有一个存储了大量属性值的数据类对象，我们可以使用这个**析构声明**将数据类对象的属性值存储到自变量中。

## 摘要

至此，本教程结束。在本教程中，我们了解了 Kotlin 数据类及其提供的功能。Kotlin 数据类减少了样板代码，并提供了获取器、设置器、toString、equals 等。自动通用功能。在下一个教程中，我们将讨论 Kotlin 中的枚举。

* * *

* * *