# Kotlin 密封类

> 原文：<https://www.studytonight.com/kotlin/kotlin-sealed-class>

在本教程中，我们将学习**Kotlin 密封类**。密封类在 Kotlin 中是新的，在 Java 中不存在。

密封类用于**表示受限类层次结构**。这意味着密封类的对象只能有一个定义的类型。让我们借助一个例子来理解它。

假设在考试中需要根据题型来声明类别。题型可以是**易**、**中**或**难**。现在我们要创建一个类**问题**，所有的问题类型都要继承这个类。我们还想确保除了这个用例，没有其他类应该继承这个 **Question** 类。

这是我们可以通过密封类实现的。一个**类只能有一些指定的子类**。当我们知道父类的所有子类型，并且不想根据我们所知道的创建子类 aprt 时，这很有用。

### 密封类和枚举类的区别

枚举也提供了类似的功能。在枚举中，我们可以在其中定义特定的类型。密封类和枚举之间的主要区别是:

*   每个枚举实例仅作为单个实例存在，而密封类的子类可以有多个实例。意思是**子类(密封类)的每个对象都可以有一个状态**。

*   在枚举中，每个实例都有相同的属性和函数集。但是在 Sealed 类中，不同的子类可以有不同的属性和函数集。

*   密封类的子类可以是不同的类型，如:数据类、对象类或普通类。

## 创建 Kotlin 密封类

使用`sealed`关键字创建密封类。需要注意的是，密封类的所有子类都必须在同一个 Kotlin 文件中声明。如果我们试图在其他 Kotlin 文件中创建一个密封类的子类，我们会得到一个错误。**封存类默认为`abstract`**。意思是**我们不能实例化密封类**。

让我们创建一个名为`Question`的密封类，并为其创建三个子类`Easy`、`Moderate`和`Difficult`:

```kt
// this is our Kotlin sealed class
sealed class Question{
    // these are child classes
    class Easy(var mark: Int, var time: Int): Question()
    class Moderate(var mark: Int, var time: Int, var hint: String): Question()
    class Difficult(var mark: Int, var time: Int, var hint: String, var solution: String): Question()
}

fun main() {
    val easyQuestion = Question.Easy(1, 1)
    println("The marks and time limit for an easy question is ${easyQuestion.mark} mark and ${easyQuestion.time} min")
}
```

一个简单问题的分数和时间限制是 1 分 1 分

正如您在上面的示例中看到的，我们可以将密封类用于这样的用例，在这些用例中，我们知道所有子类都需要什么，并且创建的类严格地服务于一个目的，就像在上面的代码示例中，我们有一个由 3 个子类继承的 Question 类，就是这样。

## Kotlin 密封类:`when`表达式

我们经常使用带有`when`表达式的密封类。因为`when`表达式保证了**密封类的所有子类都被**处理。如果我们忘记处理密封类的一个特定子类，它会给出一个错误。现在没有必要有一个`else`街区。但是只有当你把`when`作为一个表达而不是一个陈述时，它才会起作用。

让我们修改上面的例子来使用`when`表达式:

```kt
sealed class Question{
    class Easy(var mark: Int, var time: Int): Question()
    class Moderate(var mark: Int, var time: Int, var hint: String): Question()
    class Difficult(var mark: Int, var time: Int, var hint: String, var solution: String): Question()
}

fun questionDetail(question: Question): String {
    return when(question){
        is Question.Easy -> "Easy question Mark: ${question.mark} Time: ${question.time} min"
        is Question.Moderate -> "Moderate question Marks:  ${question.mark} Time: ${question.time} min and Hint: ${question.hint}"
        is Question.Difficult -> "Difficult question Marks:  ${question.mark} Time: ${question.time} min Hint: ${question.hint} and Solution: ${question.solution}"
    }
}
fun main() {
    val easyQuestion = Question.Easy(1, 1)
    val difficultQuestion = Question.Difficult(5,6,"substitution method", "x = 5, y = 3")
    println(questionDetail(easyQuestion))
    println(questionDetail(difficultQuestion))
}
```

易问号:1 次:1 分钟
难问号:5 次:6 分钟提示:代入法和解法:x = 5，y = 3

使用带有密封类子类的`when`表达式，我们可以基于密封类的子类轻松定义执行特定操作的条件。

## 摘要

在本教程中，我们了解了 Kotlin Sealed 类，它们为什么有用，如何定义密封类及其子类，以及代码示例和在 Kotlin 中使用带有密封类的`when`表达式。在下一个教程中，我们将讨论 Kotlin 泛型。

* * *

* * *