# Kotlin 空安全

> 原文：<https://www.studytonight.com/kotlin/kotlin-null-safety>

在本教程中，我们将讨论 Kotlin 最重要的特性之一:**空安全。**在编程世界中，当一个变量不引用任何东西时，它被认为是`null`。如果我们尝试使用这个变量，那么我们将得到`NullPointerException`或 NPE。

让我们看一个 Java 中发生`NullPointerException`的小例子:

```kt
public class Student {
    public static void main(String[] args) {
        Integer rollNo = null;
        Integer nextRollNo = rollNo + 1;
        System.out.println("Roll No. of next student: " + nextRollNo);
    }
}
```

线程“main”Java . lang . nullpointerexception 中的异常
在 Student.main(Student.java:4)

`NullPointerException` 被认为是一个[十亿美元的错误](https://en.wikipedia.org/wiki/Tony_Hoare#Apologies_and_retractions)，因为它导致程序突然终止。由于疏忽，`NullPointerException`可以出现在代码中的任何地方。

Kotlin 通过提供一种机制来处理`NullPointerException`来解决这个问题。Kotlin 的目标是从我们的整个代码中删除 NPE。让我们讨论一下它的功能。

## 可空和非空引用

在 Kotlin 中，类型系统区分两种类型的引用:

1.  **可空引用:**这些引用可以保存空值。

2.  **非空引用:**这些引用不能包含空值。

编译器在编译时解释这些引用。编译器默认将所有变量视为**非空引用**。

让我们用例子来理解它们:

```kt
fun main() { 
    var name:String = null   // ERROR
}
```

错误:(2，23) Kotlin: Null 不能是非 Null 类型字符串的值

这里，默认情况下，编译器将`name`变量视为非空引用。所以，我们不能给它赋值 null。当我们以后使用这个`name`变量时，它消除了获得空指针异常的机会。

为了允许变量保存空值，我们在数据类型后使用`?`:

```kt
fun main() {
    var name:String? = null    // NO ERROR
}
```

这里，我们故意允许 name 变量保存空值。但是现在**在我们尝试使用的时候可能会抛出空指针异常**。Kotlin 也将处理可空引用。如果我们试着用它作为:

```kt
fun main() {
    var name:String? = "Ninja"
    println(name.length)     // ERROR
}
```

错误:(3，17)Kotlin:只安全(？。)或非空断言(！！。)字符串类型的可空接收器上允许调用？

这里编译器不允许我们直接找到字符串的长度，因为它可能会抛出空指针异常。

现在要调用可空引用的属性和函数，我们可以执行以下操作之一:

1.  在条件中检查 null

2.  安全通话

3.  猫王接线员

4.  那个！！操作员

让我们详细讨论这些方法，看看它们如何允许我们使用可空引用，以及如何处理 NPE。

## 在条件中勾选空

如果我们想调用一个函数或在可空引用上使用一个属性(比如在`name`变量上使用`name.length`，我们可以先检查空条件，然后使用它:

```kt
fun main() {
    var name:String? = "Ninja"
    if (name != null)
        println(name.length)
}
```

5

现在编译器不会给出任何错误，因为我们已经先检查了空条件。这是处理可空引用的最基本的方法。但与此同时，在程序中处处使用也是繁琐的。

## 安全通话

如果我们想要:

*   如果`name`不为空，返回`name`的长度。

*   或者如果`name`为空，则返回空

我们可以使用安全呼叫操作符(`?.`)来实现这一点。让我们在同一个例子中使用它:

```kt
fun main() {
    var name:String? = "Ninja"
    println("Length of name is ${name?.length}")

    name = null
    println("Length of name is ${name?.length}")
}
```

名称长度为 5
名称长度为空

安全呼叫操作员类似于:

```kt
if(name != null)
    // return length
else
    // return null
```

## 猫王接线员

如果使用的变量为 null，则安全调用运算符返回 null。假设我们想要:

*   如果`name`不为空，返回`name`的长度。

*   或者如果`name`为空，则给**一个特定的值**。

我们可以使用 Elvis 运算符(`?:`)来实现这一点:

```kt
fun main() {
    var name:String? = "Ninja"
    println("Length of name is ${name?.length ?: -1}")

    name = null
    println("Length of name is ${name?.length ?: -1}")
}
```

名称长度为 5
名称长度为-1

这里我们结合了安全呼叫操作符和猫王操作符。

## `!!`操作员

`!!`运算符允许我们将**可空引用更改为非空引用**。现在，如果我们尝试使用它，它将抛出空指针异常。Kotlin 编译器不会再处理它了。

所以，如果我们试图在这种情况下使用它:

```kt
fun main() {
    var name:String? = "Ninja"
    println("Length of name is ${name!!.length}") // prints 5
}
```

会没事的。但是如果我们试图在这种情况下使用它:

```kt
fun main() {
    var name:String? = null
    println("Length of name is ${name!!.length}") // Throws NullPointerException
}
```

所以`!!`操作者要尽量避免。

## 摘要

在本教程中，我们讨论了 Kotlin 中的 Null 安全性。空安全是 Kotlin 的一个重要特性，它将我们从空指针异常中拯救出来。在下一个教程中，我们将讨论 Kotlin 中的正则表达式。

* * *

* * *