# Spring Bean 生命周期

> 原文：<https://www.studytonight.com/spring-framework/spring-bean-lifecycle>

Spring bean 的生命周期包括几个步骤，包括 Bean 初始化和 Bean 销毁。这些步骤由 Spring IOC 管理，它允许我们执行自定义初始化和结束任务。

在 Spring 中，如果我们想在 bean 初始化时和销毁 bean 对象之前执行一些任务，那么我们可以使用 Spring 的预定义接口`InitializingBean`和`DisposableBean`。这些接口提供了可用于在创建 bean 之前和之后执行任务的方法。Java 也提供注释来处理 Bean 生命周期。

有两种方法可以执行这些任务:

*   接口
*   释文

Spring `InitializingBean`接口提供了一种方法`afterPropertiesSet()`，可以用来执行初始化任务，而`DisposableBean`接口提供了一种方法`destroy()`，可以在销毁 bean 对象之前进行资源清理。

## 使用接口的 Bean 生命周期

这里，我们在项目中使用这些接口。我们的项目是一个 maven 项目，包含下面的代码文件。

**//Manager.java**

我们的 bean 类将用于执行实现。实现`Employee`接口，实现`doWork()`方法。

```java
package com.studytonight.community;

import org.springframework.beans.factory.DisposableBean;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Component;

@Component
public class Manager implements InitializingBean, DisposableBean, Employee{

	@Override
	public void afterPropertiesSet() throws Exception {

		System.out.println("Perform tasks while initializing Bean");

	}
	@Override
	public void destroy() throws Exception {

		System.out.println("Perform tasks before destroying of Bean");

	}
	@Override
	public void doWork() {
		System.out.println("Manage branch office");		
	}
}
```

**//Employee.java**

它是一个包含抽象方法 doWork()的接口，该方法将被实现的类覆盖。

```java
package com.studytonight.community;
public interface Employee {

	void doWork();

}
```

**// 银行应用.java**

它是一个配置文件，读取`applicationContext`文件，使用`getBean()`方法获取 Bean，然后根据检索到的对象调用方法。

```java
package com.studytonight.community;

import org.springframework.context.annotation.AnnotationConfigApplicationContext;
public class BankApp {

	public static void main(String[] args) {

		AnnotationConfigApplicationContext context = new AnnotationConfigApplicationContext(AppConfig.class);
		Employee employee = (Employee) context.getBean("manager");
		employee.doWork();
		context.close();
	}
}
```

//app config . Java

```java
package com.studytonight.community;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;

@Configuration
@ComponentScan("com.studytonight.community")
public class AppConfig {

} 
```

//POM . XML

这个文件包含这个项目的所有依赖项，比如 spring jars、servlet jars 等。将这些依赖项放入项目中以运行应用。

```java
<project  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://maven.apache.org/POM/4.0.0 https://maven.apache.org/xsd/maven-4.0.0.xsd">
  <modelVersion>4.0.0</modelVersion>
  <groupId>com.studytonight</groupId>
  <artifactId>springApp</artifactId>
  <version>0.0.1-SNAPSHOT</version>
  <dependencies>
		<!-- https://mvnrepository.com/artifact/org.springframework/spring-web -->
		<dependency>
			<groupId>org.springframework</groupId>
			<artifactId>spring-core</artifactId>
			<version>${spring.version}</version>
		</dependency>
		<dependency>
			<groupId>org.springframework</groupId>
			<artifactId>spring-context</artifactId>
			<version>${spring.version}</version>
		</dependency>
		<dependency>
			<groupId>javax.annotation</groupId>
			<artifactId>javax.annotation-api</artifactId>
			<version>1.3.2</version>
		</dependency>
	</dependencies>
	<properties>
		<spring.version>5.2.8.RELEASE</spring.version>
	</properties>
	<build>
		<sourceDirectory>src</sourceDirectory>
		<plugins>
			<plugin>
				<artifactId>maven-compiler-plugin</artifactId>
				<version>3.8.1</version>
				<configuration>
					<source>1.8</source>
					<target>1.8</target>
				</configuration>
			</plugin>
		</plugins>

	</build>
</project>
```

### 运行应用

成功完成项目并添加依赖项后，运行应用，您将获得如下输出。

初始化 Bean 时执行任务
管理分支机构
在销毁 Bean 前执行任务

## Spring 注释:另一种方法

Spring 提供了两个注释:`@PostConstruct`和`@PreDestroy`来执行初始化和结束任务。在这种情况下，我们不需要使用接口及其方法。这种方法非常简单，值得推荐。

**注意:**对于 Java 9 及更高版本，我们需要在项目中添加一些额外的 JARs，因为 **javax.annotation** 包已经从其默认类路径中移除。因此，将以下 JARs 添加到 pm.xml 文件中。

```java
<dependency>
	<groupId>javax.annotation</groupId>
	<artifactId>javax.annotation-api</artifactId>
	<version>1.3.2</version>
</dependency>
```

## 使用 Spring 注释的生命周期

当使用注释时，没有提供任何方法，那么我们可以使用我们自己的带有注释的方法。看，initWork()方法和 predestroy()方法是我们自己创建的，我们在这两个方法上都使用了注释，以便在 Bean 生命周期中执行它们。您可以用这个代码替换上面的示例代码(manager.java)，它将产生相同的结果。

```java
package com.studytonight.community;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import org.springframework.stereotype.Component;

@Component
public class Manager implements Employee{

	public void doWork() {
		System.out.println("Manage the branch office");

	}
	@PostConstruct
	public void initWork() {

		System.out.println("Perform tasks while initializing Bean");

	}
	@PreDestroy
	public void Predestroy() {

		System.out.println("Perform tasks before destroying of Bean");
	}
}
```

### 运行应用

成功完成项目并添加依赖项后，运行应用，您将获得如下输出。

初始化 Bean 时执行任务
管理分支机构
在销毁 Bean 前执行任务

* * *

* * *