# Spring 构造器的依赖注入

> 原文：<https://www.studytonight.com/spring-framework/spring-constructorbased-dependency-injection>

在本主题中，我们使用基于构造器的依赖注入技术通过构造器注入值，但是在继续之前，让我们先了解什么是依赖注入(DI)。

**依赖注入**是一种对象定义其依赖的技术。然后，IOC 容器在 bean 创建期间注入这些依赖关系。这个过程基本上是反过来的，也被称为控制的反过来。依赖注入使我们的代码松散耦合。它分为两大类:基于构造器的依赖注入和基于 Setter 的依赖注入。在这里，我们将通过一个例子来讨论基于构造器的 DI。

我们创建了一个基于 [Maven 的 Spring 项目](https://www.studytonight.com/spring-framework/spring-maven-project)，它包含以下文件。

*   银行应用.java

*   AppConfig.java

*   Employee.java

*   Manager.java

*   Accountant.java

*   pom.xml 文件

下面是为 Spring 应用创建的 maven 项目结构。

**项目结构:**

![](img/7835ba59bf02bb2187ecbc4032dfaf89.png)

在上述项目中创建的文件包含以下代码。请参见下面的文件。

## 文件源代码:

**// 银行应用.java**

该文件包含为我们的应用创建 IOC 容器的代码。AnnotationConfigApplicationContext 类用于为应用上下文创建对象。

```java
package com.studytonight.community;

import org.springframework.context.annotation.AnnotationConfigApplicationContext;

public class BankApp {

	public static void main(String[] args) {

		AnnotationConfigApplicationContext context = new AnnotationConfigApplicationContext(AppConfig.class);
		Manager manager = context.getBean(Manager.class);
		manager.callMetting();
		context.close();
	}
}
```

//app config . Java

这是一个 Java 的配置文件，它是我们为基于 xml 的配置示例创建的 applicationContext.xml 文件的替代文件。`@Configuration`注释表示这不是一个简单的类，而是一个配置类，`@ComponentScan`注释用于表示我们 spring 项目中的组件位置。

```java
package com.studytonight.community;

import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;

@Configuration
@ComponentScan("com.studytonight.community")
public class AppConfig {

} 
```

**//Employee.java**

这是一个包含`doWork()`抽象方法的界面`Employee`。每个实现这个接口的类都必须覆盖`doWork()`方法。

```java
package com.studytonight.community;

public interface Employee {

	void doWork();

}
```

**//Accountant.java**

这是一个使用`@Component`注解的组件类。它实现了`Employee`接口并覆盖了它的方法`doWork()`。

```java
package com.studytonight.community;
import org.springframework.stereotype.Component;
@Component
public class Accountant implements Employee{

	public Accountant() {
		System.out.println("Inside Accountant Constructor");
	}
	public void doWork() {	
		System.out.println("Audit the accounts...");
	}
}
```

**//Manager.java**

这是另一个使用`@Component`注释标记并实现`Employee`接口的组件类。在这个类中，我们实现了基于构造器的依赖注入。请参见，管理器类通过使用在管理器类构造器中实例化的会计类对象来调用会计类的方法。请看下面的例子。

```java
package com.studytonight.community;
import org.springframework.stereotype.Component;

@Component
public class Manager implements Employee{

	Accountant accountant;

	public Manager(Accountant accountant) {
		System.out.println("manager constructor");
		this.accountant = accountant;
	}
	public void doWork() {
		System.out.println("Manage the branch office");
	}

	public void callMetting() {
		accountant.doWork();
	}
}
```

//POM . XML

这个文件包含这个项目的所有依赖项，比如 spring jars、servlet jars 等。将这些依赖项放入项目中以运行应用。

```java
<project  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://maven.apache.org/POM/4.0.0 https://maven.apache.org/xsd/maven-4.0.0.xsd">
  <modelVersion>4.0.0</modelVersion>
  <groupId>com.studytonight</groupId>
  <artifactId>springApp</artifactId>
  <version>0.0.1-SNAPSHOT</version>
  <dependencies>
		<!-- https://mvnrepository.com/artifact/org.springframework/spring-web -->
		<dependency>
			<groupId>org.springframework</groupId>
			<artifactId>spring-core</artifactId>
			<version>${spring.version}</version>
		</dependency>
		<dependency>
			<groupId>org.springframework</groupId>
			<artifactId>spring-context</artifactId>
			<version>${spring.version}</version>
		</dependency>
		<dependency>
			<groupId>javax.annotation</groupId>
			<artifactId>javax.annotation-api</artifactId>
			<version>1.3.2</version>
		</dependency>
	</dependencies>
	<properties>
		<spring.version>5.2.8.RELEASE</spring.version>
	</properties>
	<build>
		<sourceDirectory>src</sourceDirectory>
		<plugins>
			<plugin>
				<artifactId>maven-compiler-plugin</artifactId>
				<version>3.8.1</version>
				<configuration>
					<source>1.8</source>
					<target>1.8</target>
				</configuration>
			</plugin>
		</plugins>

	</build>
</project>
```

### 运行应用

成功完成项目并添加依赖项后，运行应用，您将获得如下输出。

内部会计建造师
经理建造师
审核账目...

## 使用 XML 进行配置

上述项目仅使用 Java 代码进行配置。那里没有 XML 配置，但是我们也可以用 XML 代码来配置它。我们只需要创建一个文件 **applicationContext.xml** 并将其读入 **BankApp** 类。 **applicationContext.xml** 文件包含以下代码。

//application context . XML

```java
<?xml version="1.0" encoding="UTF-8"?>
<beans 
	xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
	xmlns:p="http://www.springframework.org/schema/p"
	xsi:schemaLocation="http://www.springframework.org/schema/beans  
                http://www.springframework.org/schema/beans/spring-beans-3.0.xsd">

	<bean id="accountant"
		class="com.studytonight.community.Accountant" />
	<bean id="manager" class="com.studytonight.community.Manager">
		<constructor-arg>
			<ref bean="accountant" />
		</constructor-arg>
	</bean>
</beans> 
```

## 将基元值注入构造器

除了引用变量，我们还可以向构造器中注入像 int、float 等基元值。例如，在管理器类中，我们在构造器中使用 int id 和字符串名称，并从 **applicationContext.xml** 文件中注入值。

**//Manager.java**

```java
package com.studytonight.community;
import org.springframework.stereotype.Component;

@Component
public class Manager implements Employee{
	int id;
	String name;

	public Manager(int id, String name) {
		this.id = id;
		this.name = name;
	}
	public void doWork() {
		System.out.println("Manage the branch office");
	}

	public void managerInfo() {
		System.out.println("Name: "+name+" Id: "+id);
	}
}
```

//application context . XML

```java
<?xml version="1.0" encoding="UTF-8"?>
<beans 
	xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
	xmlns:p="http://www.springframework.org/schema/p"
	xsi:schemaLocation="http://www.springframework.org/schema/beans  
                http://www.springframework.org/schema/beans/spring-beans-3.0.xsd">

	<bean id="manager" class="com.studytonight.community.Manager">
		<constructor-arg type="int" value="10021" />
		<constructor-arg type="java.lang.String" value="Ramesh" />
	</bean>
</beans> 
```

### 运行应用

成功地将这两个文件更新到项目中后，运行应用，您将获得如下输出。

名称:Ramesh Id: 10021

## 指定构造器参数名

我们还可以使用构造器参数名称来消除值的歧义，就像我们在下面的例子中所做的那样。

```java
<bean id="manager" class="com.studytonight.community.Manager">
    <constructor-arg name="id" value="10021"/>
    <constructor-arg name="name" value="Ramesh"/>
</bean>
```

* * *

* * *