# Spring `@Qualifier`注解

> 原文：<https://www.studytonight.com/spring-framework/qualifier-annotation>

Spring 中的`@Qualifier`注解用于在相同类型的 bean 对象之间区分一个 bean。

如果我们有多个相同类型的 bean，并且只想连接其中一个，那么使用**@限定符**注解和**@自动连接**来指定哪个 bean 将被连接。

如果我们没有在给定的项目中使用这个注解，那么我们会得到如下错误:

原因:org . spring framework . beans . factory . nounique beans definitionexception:没有“com . study south . community . award”类型的合格 bean 可用:期望单个匹配的 bean，但找到了 3 个:bookerAward、nationalAward、pulitzerAward

在本主题中，我们使用`@Qualifier`注解来指定依赖关系实例。该项目是一个基于 [Maven 的 Spring 项目](https://www.studytonight.com/spring-framework/spring-maven-project)，包含以下文件。

*   App.java

*   AppConfig.java

*   Award.java

*   BookerAward.java

*   国家奖 java

*   普利策奖. java

*   FictionWriter.java

*   TechnicalWriter.java

*   Writer.java

*   pom.xml 文件

### **项目结构**

![](img/4486c77b5a616a6041f2b73d31ac75e1.png)

## 文件源代码:

**//App.java**

该文件包含为我们的应用创建 IOC 容器的代码。`AnnotationConfigApplicationContext`类用于为应用上下文创建一个对象。

```java
package com.studytonight.community;

import org.springframework.context.annotation.AnnotationConfigApplicationContext;

public class App {

	public static void main(String[] args) {

		AnnotationConfigApplicationContext context = new AnnotationConfigApplicationContext(AppConfig.class);
		Writer writer = context.getBean("fictionWriter", Writer.class);
		writer.write();
		writer.getAward();
		// Close the context
		context.close();
	}
}
```

//app config . Java

这是一个 Java 配置文件，它是我们为基于 xml 的配置示例创建的 **applicationContext.xml** 文件的替代文件。`@Configuration`注解表示这不是一个简单的类，而是一个配置类，`@ComponentScan`注解用于表示我们 spring 项目中的组件位置。

```java
package com.studytonight.community;

import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;

@Configuration
@ComponentScan("com.studytonight.community")
public class AppConfig {

} 
```

**//Award.java**

这是一个包含`award()`抽象方法的界面`Award`。实现这个接口的每个类都必须覆盖奖励`()`方法。

```java
package com.studytonight.community;

public interface Award {

	void award();

}
```

**//BookerAward.java**

这是一个使用`@Component`注解标记的组件类，实现了`Award`接口。

```java
package com.studytonight.community;

import org.springframework.stereotype.Component;

@Component
public class BookerAward implements Award {

	@Override
	public void award() {

		System.out.println("You got booker prize...");
	}
}
```

**//FictionWriter.java**

这是另一个使用`@Component`注解标记并实现`Writer`接口的组件类。

```java
package com.studytonight.community;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

@Component
public class FictionWriter implements Writer {

	@Autowired
	@Qualifier("bookerAward")
	private Award award;

	@Override
	public void write() {

		System.out.println("Write Fiction Novels...");
	}

	@Override
	public void getAward() {

		award.award();
	}
}
```

**//国家奖。java**

这是另一个使用`@Component`注解标记并实现`Award`接口的组件类。

```java
package com.studytonight.community;

import org.springframework.stereotype.Component;

@Component
public class NationalAward implements Award{

	public void award() {

		System.out.println("Your Writting got National Award!");
	}
}
```

**//普利策奖。java**

这是另一个使用`@Component`注解标记并实现`Award`接口的组件类。

```java
package com.studytonight.community;

import org.springframework.stereotype.Component;

@Component
public class PulitzerAward implements Award {

	@Override
	public void award() {

		System.out.println("You have won Pulitzer Award.");
	}
}
```

**//TechnicalWriter.java**

这是另一个使用`@Component`注解标记并实现`Writer`接口的组件类。

```java
package com.studytonight.community;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

@Component
public class TechnicalWriter implements Writer{

	@Autowired
	@Qualifier("pulitzerAward")
	private Award award;

	@Override
	public void write() {

		System.out.println("Write technical stuffs...");
	}

	@Override
	public void getAward() {
		award.award();

	}
}
```

**//Writer.java**

```java
package com.studytonight.community;

public interface Writer {

	void write();
	void getAward();
}
```

//POM . XML

这个文件包含这个项目的所有依赖项，比如 spring jars、servlet jars 等。将这些依赖项放入项目中以运行应用。

```java
<project  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://maven.apache.org/POM/4.0.0 https://maven.apache.org/xsd/maven-4.0.0.xsd">
  <modelVersion>4.0.0</modelVersion>
  <groupId>com.studytonight</groupId>
  <artifactId>springproject</artifactId>
  <version>0.0.1-SNAPSHOT</version>
  <dependencies>
  <!-- https://mvnrepository.com/artifact/org.springframework/spring-web -->
  <dependency>
			<groupId>org.springframework</groupId>
			<artifactId>spring-core</artifactId>
			<version>${spring.version}</version>
		</dependency>
		<dependency>
			<groupId>org.springframework</groupId>
			<artifactId>spring-context</artifactId>
			<version>${spring.version}</version>
		</dependency>
	</dependencies>
	<properties>
		<spring.version>5.2.8.RELEASE</spring.version>
	</properties>
  <build>
    <sourceDirectory>src</sourceDirectory>
    <plugins>
      <plugin>
        <artifactId>maven-compiler-plugin</artifactId>
        <version>3.8.1</version>
        <configuration>
          <source>1.8</source>
          <target>1.8</target>
        </configuration>
      </plugin>
    </plugins>

  </build>
</project>
```

写小说...
你获得了布克奖...

* * *

* * *