# Spring REST 控制器

> 原文：<https://www.studytonight.com/spring-framework/spring-restcontroller>

RestController 是与 REST 服务一起工作的控制器的专用版本。Spring 框架传统上使用`@Controller`注释来创建处理 HTTP 请求的控制器。

在 Spring 4 中，添加了一个新的注释`@RestController`来创建一个特定的控制器，它是@Controller 和@RequestBody 注释的组合形式。它减少了用@ResponseBody 注释添加控制器的每个请求处理方法的工作量。

创建 RestController 的目的是处理 REST 服务请求。让我们通过例子来理解。在这里，我们创建了一个基于 maven 的 spring 项目，如下所示。

### Spring 支架项目结构

![](img/9b7b06760729d252c402680f83328095.png)

//app config . Java

这是一个配置文件，它是我们为基于 xml 的配置示例创建的 **applicationContext.xml** 文件的替代文件。`@Configuration`注释表示这不是一个简单的类，而是一个配置类，`@ComponentScan` 注释用于表示我们 spring 项目中组件类的位置。

```java
package com.studytonight;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.servlet.ViewResolver;
import org.springframework.web.servlet.config.annotation.EnableWebMvc;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;
import org.springframework.web.servlet.view.InternalResourceViewResolver;
@EnableWebMvc
@Configuration
@ComponentScan("com.studytonight.controllers")
public class AppConfig implements WebMvcConfigurer{

	@Autowired
	ApplicationContext applicationContext;

	@Bean
	public ViewResolver viewResolver() {
		InternalResourceViewResolver ivr = new InternalResourceViewResolver();
		ivr.setPrefix("/WEB-INF/views/");
		ivr.setSuffix(".jsp");
		ivr.setOrder(0);
		return ivr;
	}
}
```

//main app . Java

该文件包含为我们的应用创建 IOC 容器的代码。`AnnotationConfigWebApplicationContext`类用于为应用上下文创建一个对象。

```java
package com.studytonight;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRegistration;
import org.springframework.web.WebApplicationInitializer;
import org.springframework.web.context.support.AnnotationConfigWebApplicationContext;
import org.springframework.web.servlet.DispatcherServlet;
public class MainApp implements WebApplicationInitializer {

	@Override
	public void onStartup(ServletContext servletContext) throws ServletException { 
		AnnotationConfigWebApplicationContext context = new AnnotationConfigWebApplicationContext();
		context.register(AppConfig.class);
		context.setServletContext(servletContext);
		ServletRegistration.Dynamic servlet = servletContext.addServlet("dispatcher", new DispatcherServlet(context));
		servlet.setLoadOnStartup(1);
		servlet.addMapping("/");
		context.close();	
	}
}
```

**//HomeController.java**

这是控制器类，但不简单，是 RestController 有两个方法 getUser()和 postUser()，这两个方法都以 JSON 格式返回一个用户对象。为了将结果绑定到 Java 到 JSON，我们在 pom.xml 文件中使用了杰克逊库。

```java
package com.studytonight.controllers;

import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import com.studytonight.models.User;

@RestController
public class HomeController {

	@GetMapping("/user")
	public User getUser(@RequestParam(value = "name", defaultValue = "david") String name,
			@RequestParam(value = "lastName", defaultValue = "mark") String lastName,
			@RequestParam(value = "email", defaultValue = "david@gmail.com") String email) {		
		User user = new User();
		user.setName(name);
		user.setLastName(lastName);
		user.setEmail(email);
		return user;
	}

	@PostMapping("/user")
	public User postUser(@RequestBody User user) {
		return user;
	}
}
```

这是我们需要添加到 **pom.xml** 文件中的依赖项，以将数据绑定到 JSON 格式。

```java
<dependency>
    <groupId>com.fasterxml.jackson.core</groupId>
	<artifactId>jackson-databind</artifactId>
	<version>2.10.0.pr3</version>
</dependency>
```

**//User.java**

这是一个用户类，将被用作一个 bean 来保存从 HTML 表单输入的用户数据。

```java
package com.studytonight.models;
import javax.validation.constraints.Email;
import javax.validation.constraints.NotEmpty;
public class User {

	@NotEmpty(message = "Name can not empty")
	String name;
	String lastName;
	@NotEmpty(message = "Email can not empty")
	@Email
	String email;

	public String getEmail() {
		return email;
	}
	public void setEmail(String email) {
		this.email = email;
	}
	public String getName() {
		return name;
	}
	public void setName(String name) {
		this.name = name;
	}
	public String getLastName() {
		return lastName;
	}
	public void setLastName(String lastName) {
		this.lastName = lastName;
	}
}
```

### 运行应用

成功完成项目并添加依赖项后，运行应用，您将获得如下输出。

如果我们不提供参数值，那么它会显示控制器中设置的默认值。

![](img/cd701fa7108aabe43cb6316fb9548b13.png)

这就是我们如何通过使用 get request 向控制器传递新值，并以 JSON 格式获取新数据。

![](img/46d6a9a8725bd39479597dcc7a74b6d3.png)

* * *

* * *