# SASS `@mixin`和`@include`

> 原文：<https://www.studytonight.com/sass/sass-mixins-and-include>

SASS Mixins 是用于定义可以在整个样式表中重用的样式的函数，从而允许您**编写可重用的样式规则**。

> 写一次，用一次又一次。

这些和我们在其他编程语言中使用的函数是一样的，比如 **C、C++、JavaScript 等等**。您需要编写这些带有一些**样式规则**的混搭，然后您可以在样式表的其他地方多次重用它。您可以使用`@mixin` AT 规则创建混搭。

## `@mixin`at-规则的语法

以下是使用`@mixin` AT 规则定义 SASS 混合的语法:

```sass
@mixin <name> { 
    // styling rules
} 

// or 

@mixin name(<arguments...>) { 
    // styling rules
}
```

SASS mixin 可以有任何名称，您**可以在其中包含任何有效的样式语句**，就像函数定义一样。【SASS mixin 也带参数，我们将在本教程后面介绍关于 SASS mixin 的参数。

## 使用带有`@include` AT 规则的混合:

为了使用使用`@mixin`at-规则定义的混合，我们使用`@include`at-规则。以下是使用@include AT 规则来使用预定义的 SASS 混合的语法:

```sass
@include <name> 

// or 

@include <name>(<arguments...>)
```

其中**名称**是 mixin 的名称，**参数**指定 mixin 接受的参数。

让我们举一个简单的例子，了解一下在 SASS 中`@mixin`和`@include` AT 规则的用法。下面我们有一个 SASS 代码，其中我们定义了一个 mixin **基本列表**，和另一个 mixin **内联列表**。我们已经使用`@include` AT 规则在任何需要的地方调用预定义的混搭。

```sass
// defining a mixin
@mixin basic-list
  margin: 0
  padding: 0
  list-style: none

// using a mixin inside another mixin
@mixin inline-list
  @include basic-list

  li
    display: inline-block
    margin:
      left: -2px
      right: 2em

// calling a mixin in simple style rule
nav ul
  @include inline-list
```

上面的 SASS 代码将被编译成下面的 CSS 代码:

```sass
nav ul {
  margin: 0;
  padding: 0;
  list-style: none;
}
nav ul li {
  display: inline-block;
  margin-left: -2px;
  margin-right: 2em;
}
```

<u>**注:**</u> 在 SASS mixins 中，`-`(连字符)和`_`(下划线)被视为相同，这意味着**基本列表**和**基本列表**都可以用来调用**基本列表 mixin** 。

## 带参数的混合

SASS mixin 支持参数，因此您可以在调用任何 mixin 时使用它们的不同值来自定义样式规则。让我们举一个例子来定义和使用带有一些参数的 mixin。

```sass
@mixin customtextstyle($color, $size)
  color: $color
  font-size: $size
  font-weight: 400

.sidebar
  @include customtextstyle(#999999, 16px) 
```

这将被编译成以下 CSS 代码:

```sass
.sidebar {
  color: #999999;
  font-size: 16px;
  font-weight: 400
}
```

### 混合-参数的默认值

我们可以在定义混合时为参数提供默认值，这样即使用户没有为参数提供值，也会使用默认值。让我们修改上面的 SASS 代码，为`customtextstyle`混合中使用的参数提供默认值。

```sass
@mixin customtextstyle($color, $size:12px)
  color: $color
  font-size: $size
  font-weight: 400

.left-sidebar
  @include customtextstyle(#999999, 16px)

.right-sidebar
  @include customtextstyle(blue)
```

上面的 SASS 代码将被编译成下面的 CSS 代码:

```sass
.left-sidebar {
  color: #999999;
  font-size: 16px;
  font-weight: 400
}

.right-sidebar {
  color: blue;
  font-size: 12px;
  font-weight: 400
}
```

从上面的 CSS 代码中可以看到，在 CSS 类`.right-sidebar`中，**字号**属性采用了我们在定义 mixin 时提供的**默认值(12px)** 。

### 关键字参数

在调用 SASS mixin 时，我们通常按照定义 mixin 时的相同顺序提供参数。但是我们也可以在调用 mixin 时按名称传递参数值。让我们举一个简单的例子，看看这是如何实现的。

这是我们的 SASS 代码，

```sass
@mixin customtextstyle($color, $size : 12px)
  color: $color
  font-size: $size
  font-weight: 400

// calling mixin by providing argument with name
.sidebar
  @include customtextstyle(#999999, $size : 16px) 
```

它将像第一个例子一样被编译成 CSS。这里唯一的区别是，在调用 mixin 时，我们提供了参数的值和名称，这使得代码更易读。

<u>**注:**</u>**名称**应与 mixin 定义中的定义相同。因此，在更新 mixin 时要小心，如果更改参数的名称，可能会破坏代码。

### 任意参数

SASS mixin 还支持任意参数，这意味着 mixin 在被调用时可以接受任意数量的参数。如果 mixin 声明中的最后一个参数以`...`(三个点)结束，那么调用 mixin 时传递的所有额外参数都作为**列表**传递到最后一个参数中。

让我们举个例子来理解这一点。(在本例中，我们将使用`@for` AT 规则在值列表中循环)

```sass
@mixin custom-input($height, $selectors...)
  @for $i from 0 to length($selectors)
    #{nth($selectors, $i + 1)}
      padding: 10px
      height: $height
      // generating different border-radius for all
      border-radius: i*5px

@include custom-input(10px, "input.name", "input.address", "input.zip")
```

上面的代码被编译成下面的 CSS 代码:

```sass
input.name {
  padding: 10px;
  height: 10px;
  border-radius: 5px;
}

input.address {
  padding: 10px;
  height: 10px;
  border-radius: 10px;
}

input.zip {
  padding: 10px;
  height: 10px;
  border-radius: 15px;
}
```

## SASS @mixin 和@include 示例:

假设你想使用**框阴影**属性给 div 元素提供一些阴影，那么我们可以编写如下所示的 CSS。四行要设置 16 个变量的 CSS 代码，为了阴影效果添加不同规格的多个 div 元素会变得非常无聊。好吧，SASS mixins 正是把你从这种多余又无聊的工作中拯救出来的合适人选。

```sass
div
{
	-webkit-box-shadow: 0px 0px 4px #fff;
	-moz-box-shadow: 0px 0px 4px #fff;
	-ms-box-shadow: 0px 0px 4px #fff;
	box-shadow: 0px 0px 4px #fff;
} 
```

对于上面的 **CSS** ，可以编写如下的 mixins:

```sass
@mixin box-shadow($x, $y, $blur_value, $color)
{
	-webkit-box-shadow: $x, $y, $blur_value, $color;
	-moz-box-shadow: $x, $y, $blur_value, $color;
	-ms-box-shadow: $x, $y, $blur_value, $color;
	box-shadow: $x, $y, $blur_value, $color;
} 
```

现在这个 mixin 将作为一个函数，它的名字是**盒子阴影**，我们所要做的就是通过为阴影效果提供 4 个值来调用它。为了在样式表中的任何地方使用这些混合，我们使用关键字`@include`后跟混合名称。现在假设我们在样式表中使用上面创建的 mixin，它看起来如下:

```sass
div 
{
    @include box-shadow(0px, 0px, 4px, #fff);
} 
```

请注意，在上面的例子中，我们在 mixins 中传递参数，就像在其他编程语言的函数中一样。

因此，在本教程中，我们学习了所有关于 **SASS mixins 的知识，以及如何在我们的 CSS** 中随意调用它们。使用 SASS mixins 帮助我们**保持我们的 CSS 代码干净，并编写可重用的样式规则**。

* * *

* * *