# PHP 与 MySQL

> 原文：<https://www.studytonight.com/php/php-with-mysql>

PHP 是一种脚本语言，主要用于开发 web 应用程序，我们必须在数据库中搅拌用户数据。最受欢迎的开源数据库是 **MySQL** ，这是使用 PHP 最广泛的数据库之一。

在本教程中，我们将学习远程连接数据库，然后对表执行 SQL 查询。

如果你不知道如何编写 SQL 查询，你可以学习查看我们的 [SQL 教程](/dbms/sql)。

我们可以使用**MySQL**或者 **PDO** 来使用 PHP 和 MySQL。在本教程中，我们将涵盖 **PDO** 因为它支持 12 种语言，并且还支持**准备好的语句**，防止 SQL 注入。

* * *

## PDO 装置

遵循以下说明:[http://php.net/manual/en/pdo.installation.php](http://php.net/manual/en/pdo.installation.php)

* * *

## 连接到 MySQL 服务器

在执行任何查询之前，我们首先需要连接到 MySQL 服务器。下面是使用 PDO 连接到 MySQL 服务器的代码:

```php
<?php
// server details
$servername = "localhost";
$username = "username";
$password = "password";

try 
{
    $conn = new PDO("mysql:host=$servername; dbname=myDB", $username, $password);
    // set the PDO error mode to exception
    $conn->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
    echo "Connected successfully"; 
}
catch(PDOException $e)
{
    echo "Connection failed: " . $e->getMessage();
}

?>
```

在上面的代码中，您需要根据您的服务器详细信息更改**用户名**和**密码**。

* * *

## 关闭连接

完成后，关闭与 MySQL 服务器的连接是一个很好的做法。

很简单，我们只需将连接对象设置为 null，如下所示:

```php
<?php

// disconnect MySQL server connection
$conn = null;

?>
```

* * *

## 创建数据库

数据库包含一个或多个存储数据的表。

我们可以使用 PHP 在 MySQL 中创建数据库，使用 PDO 语句执行`CREATE` SQL 查询。

下面是代码:

```php
<?php

// connect to MySQL server
$servername = "localhost";
$username = "username";
$password = "password";

try 
{
    $conn = new PDO("mysql:host=$servername", $username, $password);
    // set the PDO error mode to exception
    $conn->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
    // sql query to create database
    $sql = "CREATE DATABASE myDBPDO";
    // use exec() because no results are returned
    $conn->exec($sql);
    echo "Database created successfully <br/>";
}
catch(PDOException $e)
{
    echo $sql . "<br/>" . $e->getMessage();
}

// disconnecting from MySQL server
$conn = null;

?>
```

PDO 提供异常处理，因此它比 mysqli 更好，因为在数据库上执行 SQL 查询时很有可能发生异常。

就像上面的代码一样，我们可以在变量`$sql`中提供任何 SQL 查询，并使用`exec()`函数来执行查询。

* * *

## 创建表格

就像创建数据库一样，我们也可以创建一个表，我们所要做的就是将代码中变量`$sql`中的 SQL 查询改为创建表的查询。

这里有一个示例查询来创建一个表**用户**，列有**用户标识**、**电子邮件**、**密码**和**全名**。

```php
// sql query to create table
$sql = "CREATE TABLE User (
        userid INT(6) UNSIGNED AUTO_INCREMENT PRIMARY KEY, 
        email VARCHAR(50) NOT NULL,
        password VARCHAR(50) NOT NULL,
        fullname VARCHAR(50)
        )"; 
```

这是完整的代码供您参考，

```php
<?php

// connect to MySQL server
$servername = "localhost";
$username = "username";
$password = "password";

try 
{
    $conn = new PDO("mysql:host=$servername", $username, $password);
    // set the PDO error mode to exception
    $conn->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
    // sql query to create table
    $sql = "CREATE TABLE User (
            userid INT(6) UNSIGNED AUTO_INCREMENT PRIMARY KEY, 
            email VARCHAR(50) NOT NULL,
            password VARCHAR(50) NOT NULL,
            fullname VARCHAR(50)
        )";
    // use exec() because no results are returned
    $conn->exec($sql);
    echo "Table created successfully <br/>";
}
catch(PDOException $e)
{
    echo $sql . "<br/>" . $e->getMessage();
}

// disconnecting from MySQL server
$conn = null;

?>
```

* * *

## 将数据插入表格

一旦创建了表，下一步就是将数据输入表中。要将数据输入表中，我们必须遵循某些规则:

*   如果要插入的值是字符串，我们必须用引号引起来。
*   数值不能用引号括起来。
*   `NULL`不得用引号括起来。
*   如果表格中的某一列是`AUTO_INCREMENT`，那么我们不需要在该列中插入数据，它会自动插入到表格中。

SQL 查询将是，

```php
// sql query to insert data into table
$sql = "INSERT INTO User (email, password, fullname)
        VALUES ('john@studytonight.com', 'IKilledBoogieMan', 'John Wick')"; 
```

我们所要做的就是将变量`$sql`的值更改为上面的查询，剩下的代码将保持不变，就像创建表和创建数据库一样。

* * *

## 从表中删除数据

就像我们执行 insert SQL 语句一样，我们可以执行 delete SQL 语句从表中删除数据。

```php
// sql query to delete data into table
$sql = "DELETE FROM User WHERE userid=1"; 
```

剩余的完整代码将保持不变。

* * *

## 从表中选择数据

从表中选择数据与其他操作有点不同。如果我们必须从表中选择多行数据，那么我们必须循环执行`fetch()`方法。

让我们看一个例子，我们必须从表**用户**返回所有数据。下面是 SQL 执行部分，考虑到我们已经设置了 MySQL 连接，在`$conn`连接对象中也有同样的连接。

```php
// sql query to select data from User table
$stmt = $conn->prepare("SELECT userid, email, fullname FROM User"); 
// declaring an array to store the data
$data = array();
// execute the sql statement
$stmt->execute();
// loop to fetch all the rows
while($row = $stmt->fetch(PDO::FETCH_OBJ)) 
{
    $data[] = $row;
}

// close the connection
$conn = null;
// return the data
return $data; 
```

* * *

## 结论

这不是 PHP 教程的结尾，而是开始，我们将不断添加更多的概念、示例、代码片段和教程，帮助您学习这种令人惊叹的脚本语言，敬请关注！

如有任何疑问或疑问，请在我们的 [Q & A 论坛](/studyroom)发布您的问题。

* * *

* * *